require 'uri'
require 'base64'
require 'openssl'
require 'json'
module Tradecow
  module Network
    SPOT_URL = Tradecow.spot_url
    SPOT_FUL_URL = Tradecow.spot_ful_url
    FUTURE_URL = Tradecow.future_url
    FUTURE_FUL_URL = Tradecow.future_ful_url

    def self.build_query(data)
      data.map do |k, v|
        URI.encode_www_form_component(k) << '=' << URI.encode_www_form_component(v)
      end.join('&')
    end

    def self.hash_sort(hsh)
      Hash[hsh.sort_by { |k, _| k }]
    end

    def self.sign(sign_data)
      Base64.encode64(OpenSSL::HMAC.digest('sha256', Tradecow.secret_key, sign_data)).gsub("\n", '')
    end

    def self.rest_auth_token(method, host, path, req_data = nil)
      params = {
        'AccessKeyId' => Tradecow.access_key,
        'SignatureMethod' => 'HmacSHA256',
        'SignatureVersion' => 2,
        'Timestamp' => Time.now.getutc.strftime('%Y-%m-%dT%H:%M:%S')
      }
      params = method == 'GET' && !req_data.nil? ? params.merge(req_data.stringify_keys) : params
      sign_data = "#{method}\n#{host}\n#{path}\n#{build_query(hash_sort(params))}"
      params = params.merge({ 'Signature' => sign(sign_data) })
      params
    end

    def self.url method, path, req_data = nil
      h = Tradecow::Network.rest_auth_token(method, SPOT_URL, path, req_data)
      params = Tradecow::Network.build_query(h)
      Tradecow.req_url(SPOT_FUL_URL, path, params)
    end

    def self.dm_url method, path, req_data = nil
      h = Tradecow::Network.rest_auth_token(method, FUTURE_URL, path, req_data)
      params = Tradecow::Network.build_query(h)
      Tradecow.req_url(FUTURE_FUL_URL, path, params)
    end
  end
end
# Sign
# params = Tradecow::Network.rest_auth_token('GET', 'api.huobi.pro', '/v1/account/accounts/{account-id}/balance', 'params1' => 'value1)
# Tradecow::Network.build_query(params)